/*
 * astinc.h ATARI ST kermit includes, defines, and global variable declarations
 */

#define VERSION 1      /* ATARI ST Version number  */
#define RELEASE 2      /* Update release           */
#define VERDATE "190686"

#ifndef EXTERN
#define EXTERN extern  /* global variables are extern in all but one module */
#endif





/* Symbol Definitions */

#define MAXPACKSIZ  94       /* Maximum packet size */
#define FILNAMLEN   80       /* max length of filename */
#define PFILNAMLEN  13       /* length og filename w/o path spec */
#define SOH          1       /* Start of header */
#define ETX          3       /* ^C */
#define LF          10       /* ASCII line feed */
#define CR          13       /* ASCII Carriage Return */
#define SP          32       /* ASCII space */
#define DEL        127       /* Delete (rubout) */
#define DC1         17       /* control-Q */

#define MYPCKSTART  SOH      /* packet start character */
#define MAXTRY      10       /* Times to retry a packet */
#define MYQUOTE    '#'       /* Quote character I will use */
#define MYPAD        0       /* Number of padding characters I will need */
#define MYPCHAR      0       /* Padding character I need (NULL) */
#define MYTURNCHAR  DC1      /* Usual turn around character */
#define MYEOL        CR      /* End-Of-Line character I need */
#define MYEBQ       'Y'      /* value for 8 bit prefix field */
#define MYNOEBQ     'N'      /* for no quoting! */

#define MYTIME      10       /* Seconds after which I should be timed out */
#define YOURTIME    15       /* time after I will time out */
#define MAXTIM      60       /* Maximum timeout interval */
#define MINTIM       2       /* Minumum timeout interval */

#define KER_INI      "KERMIT.OPT" /* init file name */
#define KER_LOG      "KERMIT.LOG" /* log file name */
#define KER_DEB      "KERMIT.DEB" /* debug file name */
#define KER_RSC      "KERMIT.RSC" /* resource file */


/* Macro Definitions */

/*
 * tochar: converts a control character to a printable one by adding a space.
 *
 * unchar: undoes tochar.
 *
 * ctl:      converts between control characters and printable characters by
 *      toggling the control bit (ie. ~A becomes A and A becomes ~A).
 */
#define tochar(ch)    ((ch) + ' ')
#define unchar(ch)    ((ch) - ' ')
#define ctl(ch)       ((ch) ^ 64 )


/* Parameters */

EXTERN int
   rpsiz,            /* Maximum receive packet size */
   spsiz,            /* Maximum send packet size */
   spad,             /* How much padding to send */
   rtimint,          /* Timeout for foreign host on sends */
   stimint,          /* Timeout for foreign host when waiting */
   dotimout,         /* only if true do timeout! */
   maxtry,           /* retries in case of errors (only rec. par.) */
   ibmmode,          /* do turn around character handling */
   padding,          /* do padding */
   debug,            /* whether do debugging */
   deblevel,         /* level of debugging */
   translog,         /* transaction logging */
   image,            /* eight bit transfer */
   filnamwarn;       /* generate unique filename if already present */

EXTERN int           /* could be char, but makes it easier in ASTOPT.C */
   rpcks,            /* packet start character I wait for*/
   spcks,            /* packet start character I will use */
   spadc,            /* padding character I'll send */
   reol,             /* EOL I expect */
   seol,             /* EOL I send */
   rquote,           /* quote character I expect */
   squote,           /* quote char I'll send */
   rturnchar;        /* handshake char I'll wait for */

/* Global Variables */

EXTERN int
   size,             /* Size of present data */
   n,                /* Packet number */
   n_total,          /* Total number of packets sent  */
   numtry,           /* Times this packet retried */
   oldtry,           /* Times previous packet retried */
   filecnt,          /* number of files transfered */
   bps,              /* bytes per second so far */
   nakcnt,           /* NAK receive counter */
   timcnt,           /* time out errors */
   getfile,          /* flag for get function */
   ebq;              /* eight bit quote char */

EXTERN long int
   starttrans,       /* 'universal time' of transfer start */
   startpckt;        /* 'universal time' of packet begin */


EXTERN long
   bytecnt;          /* bytes transfered since start of transfer */


EXTERN char
   state,              /* Present state of the automaton */
   recpkt[MAXPACKSIZ], /* Receive packet buffer */
   packet[MAXPACKSIZ]; /* Packet buffer */

EXTERN FILE
   *fp,              /* File pointer for current disk file */
   *deb,             /* Debug file */
   *log;             /* File pointer for Logfile */

EXTERN char
   kpathname[FILNAMLEN],    /* kermit's pathname */
   gpathname[FILNAMLEN],    /* current selected generic pathname */
   filnam[FILNAMLEN],       /* current file name */
   filnam1[FILNAMLEN];      /* converted filename */
